@@ -237,7 +237,7 @@ gdk_event_peek (void)
  * queue if event->any.window is %NULL. See gdk_display_put_event().
  **/
 void
-gdk_event_put (GdkEvent *event)
+gdk_event_put (const GdkEvent *event)
 {
   GdkDisplay *display;
   
@@ -255,7 +255,6 @@ gdk_event_put (GdkEvent *event)
   gdk_display_put_event (display, event);
 }
 
-static GMemChunk *event_chunk = NULL;
 static GHashTable *event_hash = NULL;
 
 /**
@@ -275,17 +274,10 @@ gdk_event_new (GdkEventType type)
   GdkEventPrivate *new_private;
   GdkEvent *new_event;
   
-  if (event_chunk == NULL)
-    {
-      event_chunk = g_mem_chunk_new ("events",
-				     sizeof (GdkEventPrivate),
-				     4096,
-				     G_ALLOC_AND_FREE);
-      event_hash = g_hash_table_new (g_direct_hash, NULL);
-    }
-  
-  new_private = g_chunk_new (GdkEventPrivate, event_chunk);
-  memset (new_private, 0, sizeof (GdkEventPrivate));
+  if (!event_hash)
+    event_hash = g_hash_table_new (g_direct_hash, NULL);
+
+  new_private = g_slice_new0 (GdkEventPrivate);
   
   new_private->flags = 0;
   new_private->screen = NULL;
@@ -340,7 +332,7 @@ gdk_event_new (GdkEventType type)
 }
 
 static gboolean
-gdk_event_is_allocated (GdkEvent *event)
+gdk_event_is_allocated (const GdkEvent *event)
 {
   if (event_hash)
     return g_hash_table_lookup (event_hash, event) != NULL;
@@ -359,7 +351,7 @@ gdk_event_is_allocated (GdkEvent *event)
  * gdk_event_free().
  **/
 GdkEvent*
-gdk_event_copy (GdkEvent *event)
+gdk_event_copy (const GdkEvent *event)
 {
   GdkEventPrivate *new_private;
   GdkEvent *new_event;
@@ -446,8 +438,6 @@ gdk_event_free (GdkEvent *event)
 {
   g_return_if_fail (event != NULL);
 
-  g_assert (event_chunk != NULL); /* paranoid */
-  
   if (event->any.window)
     g_object_unref (event->any.window);
   
@@ -475,8 +465,7 @@ gdk_event_free (GdkEvent *event)
 
     case GDK_BUTTON_PRESS:
     case GDK_BUTTON_RELEASE:
-      if (event->button.axes)
-	g_free (event->button.axes);
+      g_free (event->button.axes);
       break;
       
     case GDK_EXPOSE:
@@ -485,8 +474,7 @@ gdk_event_free (GdkEvent *event)
       break;
       
     case GDK_MOTION_NOTIFY:
-      if (event->motion.axes)
-	g_free (event->motion.axes);
+      g_free (event->motion.axes);
       break;
       
     case GDK_SETTING:
@@ -498,7 +486,7 @@ gdk_event_free (GdkEvent *event)
     }
 
   g_hash_table_remove (event_hash, event);
-  g_mem_chunk_free (event_chunk, event);
+  g_slice_free (GdkEventPrivate, (GdkEventPrivate*) event);
 }
 
 /**
@@ -511,7 +499,7 @@ gdk_event_free (GdkEvent *event)
  * Return value: time stamp field from @event
  **/
 guint32
-gdk_event_get_time (GdkEvent *event)
+gdk_event_get_time (const GdkEvent *event)
 {
   if (event)
     switch (event->type)
@@ -582,8 +570,8 @@ gdk_event_get_time (GdkEvent *event)
  * Return value: %TRUE if there was a state field in the event 
  **/
 gboolean
-gdk_event_get_state (GdkEvent        *event,
-                     GdkModifierType *state)
+gdk_event_get_state (const GdkEvent        *event,
+                     GdkModifierType       *state)
 {
   g_return_val_if_fail (state != NULL, FALSE);
   
@@ -658,9 +646,9 @@ gdk_event_get_state (GdkEvent        *event,
  * Return value: %TRUE if the event delivered event window coordinates
  **/
 gboolean
-gdk_event_get_coords (GdkEvent *event,
-		      gdouble  *x_win,
-		      gdouble  *y_win)
+gdk_event_get_coords (const GdkEvent *event,
+		      gdouble        *x_win,
+		      gdouble        *y_win)
 {
   gdouble x = 0, y = 0;
   gboolean fetched = TRUE;
@@ -717,9 +705,9 @@ gdk_event_get_coords (GdkEvent *event,
  * Return value: %TRUE if the event delivered root window coordinates
  **/
 gboolean
-gdk_event_get_root_coords (GdkEvent *event,
-			   gdouble  *x_root,
-			   gdouble  *y_root)
+gdk_event_get_root_coords (const GdkEvent *event,
+			   gdouble        *x_root,
+			   gdouble        *y_root)
 {
   gdouble x = 0, y = 0;
   gboolean fetched = TRUE;
@@ -782,9 +770,9 @@ gdk_event_get_root_coords (GdkEvent *event,
  * Return value: %TRUE if the specified axis was found, otherwise %FALSE
  **/
 gboolean
-gdk_event_get_axis (GdkEvent   *event,
-		    GdkAxisUse  axis_use,
-		    gdouble    *value)
+gdk_event_get_axis (const GdkEvent *event,
+		    GdkAxisUse      axis_use,
+		    gdouble        *value)
 {
   gdouble *axes;
   GdkDevice *device;
@@ -844,6 +832,37 @@ gdk_event_get_axis (GdkEvent   *event,
   return gdk_device_get_axis (device, axes, axis_use, value);
 }
 
+/**
+ * gdk_event_request_motions:
+ * @event: a valid #GdkEvent
+ *
+ * Request more motion notifies if @event is a motion notify hint event.
+ * This function should be used instead of gdk_window_get_pointer() to
+ * request further motion notifies, because it also works for extension
+ * events where motion notifies are provided for devices other than the
+ * core pointer. Coordinate extraction, processing and requesting more
+ * motion events from a %GDK_MOTION_NOTIFY event usually works like this:
+ *
+ * |[
+ * { 
+ *   /&ast; motion_event handler &ast;/
+ *   x = motion_event->x;
+ *   y = motion_event->y;
+ *   /&ast; handle (x,y) motion &ast;/
+ *   gdk_event_request_motions (motion_event); /&ast; handles is_hint events &ast;/
+ * }
+ * ]|
+ *
+ * Since: 2.12
+ **/
+void
+gdk_event_request_motions (const GdkEventMotion *event)
+{
+  g_return_if_fail (event != NULL);
+  if (event->type == GDK_MOTION_NOTIFY && event->is_hint)
+    gdk_device_get_state (event->device, event->window, NULL, NULL);
+}
+
 /**
  * gdk_event_set_screen:
  * @event: a #GdkEvent
@@ -885,7 +904,7 @@ gdk_event_set_screen (GdkEvent  *event,
  * Since: 2.2
  **/
 GdkScreen *
-gdk_event_get_screen (GdkEvent *event)
+gdk_event_get_screen (const GdkEvent *event)
 {
   if (gdk_event_is_allocated (event))
     {
@@ -1200,7 +1219,7 @@ gdk_event_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("GdkEvent",
+    our_type = g_boxed_type_register_static (g_intern_static_string ("GdkEvent"),
 					     (GBoxedCopyFunc)gdk_event_copy,
 					     (GBoxedFreeFunc)gdk_event_free);
   return our_type;
