@@ -1,5 +1,6 @@
 /* GDK - The GIMP Drawing Kit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+ * Copyright (C) 1995-2007 Peter Mattis, Spencer Kimball,
+ * Josh MacDonald, Ryan Lortie
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -904,8 +905,8 @@ gdk_window_get_state (GdkWindow *window)
  * 
  **/
 void
-gdk_window_begin_paint_rect (GdkWindow    *window,
-			     GdkRectangle *rectangle)
+gdk_window_begin_paint_rect (GdkWindow          *window,
+                             const GdkRectangle *rectangle)
 {
   GdkRegion *region;
 
@@ -967,8 +968,8 @@ gdk_window_begin_paint_rect (GdkWindow    *window,
  * 
  **/
 void	      
-gdk_window_begin_paint_region (GdkWindow *window,
-			       GdkRegion *region)
+gdk_window_begin_paint_region (GdkWindow       *window,
+                               const GdkRegion *region)
 {
 #ifdef USE_BACKING_STORE
   GdkWindowObject *private = (GdkWindowObject *)window;
@@ -1042,6 +1043,7 @@ gdk_window_end_paint (GdkWindow *window)
 {
 #ifdef USE_BACKING_STORE
   GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkWindowObject *composited;
   GdkWindowPaint *paint;
   GdkGC *tmp_gc;
   GdkRectangle clip_box;
@@ -1094,6 +1096,34 @@ gdk_window_end_paint (GdkWindow *window)
   g_object_unref (paint->pixmap);
   gdk_region_destroy (paint->region);
   g_free (paint);
+
+  /* find a composited window in our hierarchy to signal its
+   * parent to redraw, calculating the clip box as we go...
+   *
+   * stop if parent becomes NULL since then we'd have nowhere
+   * to draw (ie: 'composited' will always be non-NULL here).
+   */
+  for (composited = private;
+       composited->parent;
+       composited = composited->parent)
+    {
+      int width, height;
+
+      gdk_drawable_get_size (GDK_DRAWABLE (composited->parent),
+			     &width, &height);
+
+      clip_box.x += composited->x;
+      clip_box.y += composited->y;
+      clip_box.width = MIN (clip_box.width, width - clip_box.x);
+      clip_box.height = MIN (clip_box.height, height - clip_box.y);
+
+      if (composited->composited)
+	{
+	  gdk_window_invalidate_rect (GDK_WINDOW (composited->parent),
+				      &clip_box, FALSE);
+	  break;
+	}
+    }
 #endif /* USE_BACKING_STORE */
 }
 
@@ -2260,10 +2290,22 @@ gdk_window_update_idle (gpointer data)
   return FALSE;
 }
 
+static gboolean
+gdk_window_is_toplevel_frozen (GdkWindow *window)
+{
+  GdkWindowObject *toplevel;
+
+  toplevel = (GdkWindowObject *)gdk_window_get_toplevel (window);
+
+  return toplevel->update_and_descendants_freeze_count > 0;
+}
+
 static void
 gdk_window_schedule_update (GdkWindow *window)
 {
-  if (window && GDK_WINDOW_OBJECT (window)->update_freeze_count)
+  if (window &&
+      (GDK_WINDOW_OBJECT (window)->update_freeze_count ||
+       gdk_window_is_toplevel_frozen (window)))
     return;
 
   if (!update_idle)
@@ -2393,11 +2435,15 @@ gdk_window_process_all_updates (void)
     {
       GdkWindowObject *private = (GdkWindowObject *)tmp_list->data;
       
-      if (private->update_freeze_count)
-	update_windows = g_slist_prepend (update_windows, private);
-      else
-	gdk_window_process_updates_internal (tmp_list->data);
-      
+      if (!GDK_WINDOW_DESTROYED (tmp_list->data))
+        {
+	  if (private->update_freeze_count ||
+	      gdk_window_is_toplevel_frozen (tmp_list->data))
+	    update_windows = g_slist_prepend (update_windows, private);
+	  else
+	    gdk_window_process_updates_internal (tmp_list->data);
+	}
+
       g_object_unref (tmp_list->data);
       tmp_list = tmp_list->next;
     }
@@ -2441,7 +2487,9 @@ gdk_window_process_updates (GdkWindow *window,
       return;
     }
   
-  if (private->update_area && !private->update_freeze_count)
+  if (private->update_area &&
+      !private->update_freeze_count &&
+      !gdk_window_is_toplevel_frozen (window))
     {      
       gdk_window_process_updates_internal (window);
       update_windows = g_slist_remove (update_windows, window);
@@ -2461,18 +2509,18 @@ gdk_window_process_updates (GdkWindow *window,
 /**
  * gdk_window_invalidate_rect:
  * @window: a #GdkWindow
- * @rect: rectangle to invalidate
+ * @rect: rectangle to invalidate or %NULL to invalidate the whole
+ *      window
  * @invalidate_children: whether to also invalidate child windows
  *
  * A convenience wrapper around gdk_window_invalidate_region() which
  * invalidates a rectangular region. See
  * gdk_window_invalidate_region() for details.
- * 
  **/
 void
-gdk_window_invalidate_rect   (GdkWindow    *window,
-			      GdkRectangle *rect,
-			      gboolean      invalidate_children)
+gdk_window_invalidate_rect (GdkWindow          *window,
+                            const GdkRectangle *rect,
+                            gboolean            invalidate_children)
 {
   GdkRectangle window_rect;
   GdkRegion *region;
@@ -2503,8 +2551,8 @@ gdk_window_invalidate_rect   (GdkWindow    *window,
 }
 
 static void
-draw_ugly_color (GdkWindow *window,
-                GdkRegion *region)
+draw_ugly_color (GdkWindow       *window,
+                 const GdkRegion *region)
 {
   /* Draw ugly color all over the newly-invalid region */
   GdkColor ugly_color = { 0, 50000, 10000, 10000 };
@@ -2552,9 +2600,10 @@ draw_ugly_color (GdkWindow *window,
  * invalidated.
  **/
 void
-gdk_window_invalidate_maybe_recurse (GdkWindow *window,
-				     GdkRegion *region,
-				     gboolean (*child_func) (GdkWindow *, gpointer),
+gdk_window_invalidate_maybe_recurse (GdkWindow       *window,
+                                     const GdkRegion *region,
+                                     gboolean       (*child_func) (GdkWindow *,
+                                                                   gpointer),
 				     gpointer   user_data)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
@@ -2601,19 +2650,22 @@ gdk_window_invalidate_maybe_recurse (GdkWindow *window,
 	  child_region = gdk_region_rectangle (&child_rect);
 	  
 	  /* remove child area from the invalid area of the parent */
-	  if (GDK_WINDOW_IS_MAPPED (child) && !child->shaped)
+	  if (GDK_WINDOW_IS_MAPPED (child) && !child->shaped &&
+	      !child->composited)
 	    gdk_region_subtract (visible_region, child_region);
 	  
 	  if (child_func && (*child_func) ((GdkWindow *)child, user_data))
 	    {
-	      gdk_region_offset (region, - child_rect.x, - child_rect.y);
+              GdkRegion *tmp = gdk_region_copy (region);
+
+	      gdk_region_offset (tmp, - child_rect.x, - child_rect.y);
 	      gdk_region_offset (child_region, - child_rect.x, - child_rect.y);
-	      gdk_region_intersect (child_region, region);
+	      gdk_region_intersect (child_region, tmp);
 	      
 	      gdk_window_invalidate_maybe_recurse ((GdkWindow *)child,
 						   child_region, child_func, user_data);
 	      
-	      gdk_region_offset (region, child_rect.x, child_rect.y);
+	      gdk_region_destroy (tmp);
 	    }
 
 	  gdk_region_destroy (child_region);
@@ -2675,9 +2727,9 @@ true_predicate (GdkWindow *window,
  * fine grained control over which children are invalidated.
  **/
 void
-gdk_window_invalidate_region (GdkWindow *window,
-			      GdkRegion *region,
-			      gboolean   invalidate_children)
+gdk_window_invalidate_region (GdkWindow       *window,
+			      const GdkRegion *region,
+			      gboolean         invalidate_children)
 {
   gdk_window_invalidate_maybe_recurse (window, region,
 				       invalidate_children ?
@@ -2784,6 +2836,58 @@ gdk_window_thaw_updates (GdkWindow *window)
     gdk_window_schedule_update (window);
 }
 
+/**
+ * gdk_window_freeze_toplevel_updates_libgtk_only:
+ * @window: a #GdkWindow
+ *
+ * Temporarily freezes a window and all its descendants such that it won't
+ * receive expose events.  The window will begin receiving expose events
+ * again when gdk_window_thaw_toplevel_updates_libgtk_only() is called. If
+ * gdk_window_freeze_toplevel_updates_libgtk_only()
+ * has been called more than once,
+ * gdk_window_thaw_toplevel_updates_libgtk_only() must be called
+ * an equal number of times to begin processing exposes.
+ *
+ * This function is not part of the GDK public API and is only
+ * for use by GTK+.
+ **/
+void
+gdk_window_freeze_toplevel_updates_libgtk_only (GdkWindow *window)
+{
+  GdkWindowObject *private = (GdkWindowObject *)window;
+
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (private->window_type != GDK_WINDOW_CHILD);
+
+  private->update_and_descendants_freeze_count++;
+}
+
+/**
+ * gdk_window_thaw_toplevel_updates_libgtk_only:
+ * @window: a #GdkWindow
+ * 
+ * Thaws a window frozen with
+ * gdk_window_freeze_toplevel_updates_libgtk_only().
+ *
+ * This function is not part of the GDK public API and is only
+ * for use by GTK+.
+ **/
+void
+gdk_window_thaw_toplevel_updates_libgtk_only (GdkWindow *window)
+{
+  GdkWindowObject *private = (GdkWindowObject *)window;
+
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  g_return_if_fail (private->window_type != GDK_WINDOW_CHILD);
+  g_return_if_fail (private->update_and_descendants_freeze_count > 0);
+
+  private->update_and_descendants_freeze_count--;
+
+  gdk_window_schedule_update (window);
+}
+
 /**
  * gdk_window_set_debug_updates:
  * @setting: %TRUE to turn on update debugging
@@ -2947,9 +3051,12 @@ gdk_window_constrain_size (GdkGeometry *geometry,
 /**
  * gdk_window_get_pointer:
  * @window: a #GdkWindow
- * @x: return location for X coordinate of pointer
- * @y: return location for Y coordinate of pointer
- * @mask: return location for modifier mask
+ * @x: return location for X coordinate of pointer or %NULL to not
+ *      return the X coordinate
+ * @y: return location for Y coordinate of pointer or %NULL to not
+ *      return the Y coordinate
+ * @mask: return location for modifier mask or %NULL to not return the
+ *      modifier mask
  *
  * Obtains the current pointer position and modifier state.
  * The position is given in coordinates relative to the upper left 
@@ -3056,5 +3163,65 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   return gdk_window_foreign_new_for_display (gdk_display_get_default (), anid);
 }
 
+/**
+ * gdk_window_set_composited:
+ * @window: a #GdkWindow
+ * @composited: %TRUE to set the window as composited
+ *
+ * Sets a #GdkWindow as composited, or unsets it. Composited 
+ * windows do not automatically have their contents drawn to 
+ * the screen. Drawing is redirected to an offscreen buffer 
+ * and an expose event is emitted on the parent of the composited 
+ * window. It is the responsibility of the parent's expose handler
+ * to manually merge the off-screen content onto the screen in
+ * whatever way it sees fit. See <xref linkend="composited-window-example"/>
+ * for an example.
+ *
+ * It only makes sense for child windows to be composited; see
+ * gdk_window_set_opacity() if you need translucent toplevel
+ * windows.
+ *
+ * An additional effect of this call is that the area of this
+ * window is no longer clipped from regions marked for
+ * invalidation on its parent. Draws done on the parent
+ * window are also no longer clipped by the child.
+ *
+ * This call is only supported on some systems (currently,
+ * only X11 with new enough Xcomposite and Xdamage extensions). 
+ * You must call gdk_display_supports_composite() to check if
+ * setting a window as composited is supported before
+ * attempting to do so.
+ *
+ * Since: 2.12
+ */
+void
+gdk_window_set_composited (GdkWindow *window,
+                           gboolean   composited)
+{
+  GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkDisplay *display;
+
+  g_return_if_fail (window != NULL);
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  composited = composited != FALSE;
+
+  if (private->composited == composited)
+    return;
+
+  display = gdk_drawable_get_display (GDK_DRAWABLE (window));
+
+  if (!gdk_display_supports_composite (display) && composited)
+    {
+      g_warning ("gdk_window_set_composited called but "
+		 "compositing is not supported");
+      return;
+    }
+
+  _gdk_windowing_window_set_composited (window, composited);
+
+  private->composited = composited;
+}
+
 #define __GDK_WINDOW_C__
 #include "gdkaliasdef.c"
