@@ -167,6 +167,17 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
 
   class->owner_change = gtk_clipboard_owner_change;
 
+  /**
+   * GtkClipboard::owner-change:
+   * @clipboard: the #GtkClipboard on which the signal is emitted
+   * @event: the @GdkEventOwnerChange event 
+   *
+   * The ::owner-change signal is emitted when GTK+ receives an
+   * event that indicates that the ownership of the selection 
+   * associated with @clipboard has changed.
+   *
+   * Since: 2.6
+   */ 
   clipboard_signals[OWNER_CHANGE] =
     g_signal_new (I_("owner_change"),
 		  G_TYPE_FROM_CLASS (gobject_class),
@@ -182,26 +193,31 @@ static void
 gtk_clipboard_finalize (GObject *object)
 {
   GtkClipboard *clipboard;
-  GtkWidget *clipboard_widget;
-  GSList *clipboards;
+  GtkWidget *clipboard_widget = NULL;
+  GSList *clipboards = NULL;
 
   clipboard = GTK_CLIPBOARD (object);
 
-  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
-  if (g_slist_index (clipboards, clipboard) >= 0)
-    g_warning ("GtkClipboard prematurely finalized");
+  if (clipboard->display)
+    {
+      clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
 
-  /*  don't use get_clipboard_widget() here because it would create the
-   *  widget if it doesn't exist.
-   */
-  clipboard_widget = g_object_get_data (G_OBJECT (clipboard->display),
-                                        "gtk-clipboard-widget");
+      if (g_slist_index (clipboards, clipboard) >= 0)
+        g_warning ("GtkClipboard prematurely finalized");
+
+      clipboards = g_slist_remove (clipboards, clipboard);
+
+      g_object_set_data (G_OBJECT (clipboard->display), "gtk-clipboard-list", 
+                         clipboards);
+
+      /* don't use get_clipboard_widget() here because it would create the
+       * widget if it doesn't exist.
+       */
+      clipboard_widget = g_object_get_data (G_OBJECT (clipboard->display),
+                                            "gtk-clipboard-widget");
+    }
 
   clipboard_unset (clipboard);
-  
-  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
-  clipboards = g_slist_remove (clipboards, clipboard);
-  g_object_set_data (G_OBJECT (clipboard->display), I_("gtk-clipboard-list"), clipboards);
 
   if (clipboard->store_loop && g_main_loop_is_running (clipboard->store_loop))
     g_main_loop_quit (clipboard->store_loop);
@@ -1726,7 +1742,7 @@ gtk_clipboard_wait_is_target_available (GtkClipboard *clipboard,
  * _gtk_clipboard_handle_event:
  * @event: a owner change event
  * 
- * Emits the ::owner_change signal on the appropriate @clipboard.
+ * Emits the #GtkClipboard::owner-change signal on the appropriate @clipboard.
  *
  * Since: 2.6
  **/
